use leptos::*;
use leptos_meta::*;
use leptos_router::*;
use serde::Deserialize;

pub fn shell(_options: LeptosOptions) -> impl IntoView {
    view! {
        <!DOCTYPE html>
        <html lang="en">
            <head>
                <meta charset="utf-8"/>
                <meta name="viewport" content="width=device-width, initial-scale=1"/>
            </head>
            <body>
                <App/>
            </body>
        </html>
    }
}

#[component]
pub fn App() -> impl IntoView {
    // Provides context that manages stylesheets, titles, meta tags, etc.
    provide_meta_context();

    view! {
        // injects a stylesheet into the document <head>
        // id=leptos means cargo-leptos will hot-reload this stylesheet
        <Stylesheet id="leptos" href="/pkg/tiny-intranet.css"/>

        // sets the document title
        <Title text="Welcome to Leptos"/>

        // content for this welcome page
        <Router>
            <main>
                <Routes>
                    <Route path="" view=HomePage/>
                </Routes>
            </main>
        </Router>
    }
}

/// Renders the home page of your application.
#[component]
fn HomePage() -> impl IntoView {
    let host = create_resource(|| {}, |_| async move { get_host().await });

    view! {
        <h1>"Welcome to Leptos!"</h1>
        <Suspense fallback=|| view! { "Loading..." }>
        <p>Host: {host.get().map(format_result)}</p>
        </Suspense>
    }
}

fn format_result(r: Result<String, ServerFnError>) -> impl IntoView {
    // TODO if host is status.client, check cookie and show action form
    view! {
        <ErrorBoundary fallback=|_| view! { "error" }>
        {r}
        </ErrorBoundary>
    }
}

#[derive(Deserialize, Debug)]
struct HostOverride {
    host: Option<String>,
}

#[server(GetHost)]
async fn get_host() -> Result<String, ServerFnError> {
    use axum::extract::Query;
    use axum::http::HeaderMap;
    use leptos_axum::extract;

    let (headers, query): (HeaderMap, Query<HostOverride>) = extract().await?;
    eprintln!("Headers {headers:?}");
    let host = query
        .host
        .clone()
        .or_else(|| {
            headers.get("host").and_then(|v| {
                eprintln!("Host {v:?}");
                v.to_str().ok().map(str::to_owned)
            })
        })
        .unwrap_or_default();

    Ok(host)
}
